@@ -54,16 +54,15 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <libbonobo.h>
 #include <gconf/gconf-client.h>
 #include <libgnomeui/gnome-help.h>
-#include <libgnome/gnome-init.h>
 #include <libgnomeui/gnome-url.h>
 
 #ifdef HAVE_LIBECAL
 #include "calendar-client.h"
 #endif
 
+#define CLOCK_ICON "gnome-panel-clock"
 #define INTERNETSECOND (864)
 #define INTERNETBEAT   (86400)
 
@@ -192,7 +191,6 @@ unfix_size (ClockData *cd)
 
 static int
 calculate_minimum_width (GtkWidget   *widget,
-			 GtkWidget   *button,
 			 const gchar *text)
 {
 	PangoContext *context;
@@ -217,13 +215,6 @@ calculate_minimum_width (GtkWidget   *widget,
 
 	width += 2 * (focus_width + focus_pad + widget->style->xthickness);
 
-	gtk_widget_style_get (button,
-			      "focus-line-width", &focus_width,
-			      "focus-padding", &focus_pad,
-			      NULL);
-
-	width += 2 * (focus_width + focus_pad + button->style->xthickness);
-
 	return width;
 }
 
@@ -330,7 +321,7 @@ calculate_minimum_height (GtkWidget        *widget,
 static gboolean
 use_two_line_format (ClockData *cd)
 {
-        if (cd->size >= 2 * calculate_minimum_height (cd->applet, cd->orient))
+        if (cd->size >= 2 * calculate_minimum_height (cd->toggle, cd->orient))
                 return TRUE;
 
         return FALSE;
@@ -680,10 +671,9 @@ format_time (ClockFormat format,
         }
 
         strftime (result, sizeof (result), time_format, tm);
-
         g_free (time_format);
 
-        return g_strdup (result);
+        return g_locale_to_utf8 (result, -1, NULL, NULL, NULL);
 }
 
 static void
@@ -1294,6 +1284,8 @@ calendar_day_activated (ClockData   *cd,
 					"calendar:///?startdate=%.4d%.2d%.2d",
 					year, month + 1, day);
 
+	error = NULL;
+
 	if (!gdk_spawn_command_line_on_screen (gtk_widget_get_screen (cd->calendar),
 					       command_line, &error)) {
 		g_printerr ("Cannot launch calendar: %s\n", error->message);
@@ -1442,6 +1434,7 @@ create_calendar (ClockData *cd,
         struct tm                 *tm;
 
 	window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
+	gtk_window_set_icon_name (window, CLOCK_ICON);
         gtk_window_set_screen (window, screen);
 
         frame = gtk_frame_new (NULL);
@@ -1754,13 +1747,13 @@ update_orient (ClockData *cd)
 	gdouble      angle;
 
 	text = gtk_label_get_text (GTK_LABEL (cd->clockw));
-	min_width = calculate_minimum_width (cd->applet, cd->toggle, text);
+	min_width = calculate_minimum_width (cd->toggle, text);
 
 	if (cd->orient == PANEL_APPLET_ORIENT_LEFT &&
-	    min_width > cd->applet->allocation.width)
+	    min_width > cd->toggle->allocation.width)
 		new_angle = 270;
 	else if (cd->orient == PANEL_APPLET_ORIENT_RIGHT &&
-		 min_width > cd->applet->allocation.width)
+		 min_width > cd->toggle->allocation.width)
 		new_angle = 90;
 	else
 		new_angle = 0;
@@ -1790,7 +1783,7 @@ applet_change_orient (PanelApplet       *applet,
 
 /* this is when the panel size changes */
 static void
-applet_change_pixel_size (PanelApplet	*applet,
+toggle_change_pixel_size (GtkWidget     *widget,
 			  GtkAllocation *allocation,
 			  ClockData	*cd)
 {
@@ -1949,7 +1942,8 @@ try_config_tool (GdkScreen  *screen,
 		
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (gtk_widget_destroy), NULL);
-			
+
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), CLOCK_ICON);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 			
@@ -1986,6 +1980,7 @@ config_date (BonoboUIComponent *uic,
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (gtk_widget_destroy), NULL);
 		
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), CLOCK_ICON);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 
@@ -2351,7 +2346,9 @@ fill_clock_applet (PanelApplet *applet)
 	gtk_container_set_border_width (GTK_CONTAINER (cd->toggle), 0);
 	gtk_container_add (GTK_CONTAINER (cd->applet), cd->toggle);
 
-	gtk_window_set_default_icon_name ("gnome-panel-clock");
+#ifndef CLOCK_INPROCESS
+	gtk_window_set_default_icon_name (CLOCK_ICON);
+#endif
 	gtk_widget_show (cd->applet);
 
 	/* FIXME: Update this comment. */
@@ -2363,9 +2360,9 @@ fill_clock_applet (PanelApplet *applet)
 			  G_CALLBACK (applet_change_orient),
 			  cd);
 
-	g_signal_connect (G_OBJECT (cd->applet),
+	g_signal_connect (G_OBJECT (cd->toggle),
 			  "size_allocate",
-			  G_CALLBACK (applet_change_pixel_size),
+			  G_CALLBACK (toggle_change_pixel_size),
 			  cd);
 
 	panel_applet_set_background_widget (PANEL_APPLET (cd->applet),
@@ -2589,6 +2586,8 @@ properties_response_cb (GtkWidget *widget,
 					  G_CALLBACK (gtk_widget_destroy),
 					  NULL);
 
+			gtk_window_set_icon_name (GTK_WINDOW (dialog),
+						  CLOCK_ICON);
 			gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 			gtk_window_set_screen (GTK_WINDOW (dialog),
 					       gtk_widget_get_screen (cd->applet));
@@ -2624,6 +2623,7 @@ display_properties_dialog (BonoboUIComponent *uic,
 						 GTK_RESPONSE_CLOSE,
 						 NULL);
 
+	gtk_window_set_icon_name (GTK_WINDOW (cd->props), CLOCK_ICON);
 	gtk_dialog_set_has_separator (GTK_DIALOG (cd->props), FALSE);
 	gtk_dialog_set_default_response (GTK_DIALOG (cd->props), GTK_RESPONSE_CLOSE);
 	gtk_window_set_resizable (GTK_WINDOW (cd->props), FALSE);
@@ -2763,6 +2763,7 @@ display_help_dialog (BonoboUIComponent *uic,
 				  G_CALLBACK (gtk_widget_destroy),
 				  NULL);
 
+		gtk_window_set_icon_name (GTK_WINDOW (dialog), CLOCK_ICON);
 		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 		gtk_window_set_screen (GTK_WINDOW (dialog),
 				       gtk_widget_get_screen (cd->applet));
@@ -2804,9 +2805,10 @@ display_about_dialog (BonoboUIComponent *uic,
 		      "authors", authors,
 		      "documenters", documenters,
 		      "translator-credits", _("translator-credits"),
-		      "logo-icon-name", "gnome-panel-clock",
+		      "logo-icon-name", CLOCK_ICON,
 		      NULL);
 	
+	gtk_window_set_icon_name (GTK_WINDOW (cd->about), CLOCK_ICON);
 	gtk_window_set_wmclass (GTK_WINDOW (cd->about), "clock", "Clock");
 	gtk_window_set_screen (GTK_WINDOW (cd->about),
 			       gtk_widget_get_screen (cd->applet));
@@ -2834,9 +2836,17 @@ clock_factory (PanelApplet *applet,
 	return retval;
 }
 
+#ifdef CLOCK_INPROCESS
+PANEL_APPLET_BONOBO_SHLIB_FACTORY ("OAFIID:GNOME_ClockApplet_Factory",
+				   PANEL_TYPE_APPLET,
+				   "ClockApplet",
+				   clock_factory,
+				   NULL);
+#else
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_ClockApplet_Factory",
                              PANEL_TYPE_APPLET,
                              "ClockApplet",
                              "0",
                              clock_factory,
                              NULL);
+#endif
