@@ -311,7 +311,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_STRUCT_OFFSET (PanelWidgetClass, size_change),
                               NULL,
                               NULL, 
-                              panel_marshal_VOID__VOID,
+                              g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
 
@@ -322,7 +322,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_STRUCT_OFFSET (PanelWidgetClass, back_change),
                               NULL,
                               NULL, 
-                              panel_marshal_VOID__VOID,
+                              g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
 
@@ -333,7 +333,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_STRUCT_OFFSET (PanelWidgetClass, applet_move),
                               NULL,
                               NULL, 
-                              panel_marshal_VOID__POINTER,
+                              g_cclosure_marshal_VOID__POINTER,
                               G_TYPE_NONE,
                               1,
                               G_TYPE_POINTER); 
@@ -345,7 +345,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_STRUCT_OFFSET (PanelWidgetClass, applet_added),
                               NULL,
                               NULL, 
-                              panel_marshal_VOID__POINTER,
+                              g_cclosure_marshal_VOID__POINTER,
                               G_TYPE_NONE,
                               1,
                               G_TYPE_POINTER); 
@@ -357,7 +357,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_STRUCT_OFFSET (PanelWidgetClass, applet_removed),
                               NULL,
                               NULL, 
-                              panel_marshal_VOID__POINTER,
+                              g_cclosure_marshal_VOID__POINTER,
                               G_TYPE_NONE,
                               1,
                               G_TYPE_POINTER); 
@@ -405,7 +405,7 @@ panel_widget_class_init (PanelWidgetClass *class)
 			      G_STRUCT_OFFSET (PanelWidgetClass, tab_move),
                               NULL,
                               NULL,
-                              panel_marshal_VOID__BOOLEAN,
+                              g_cclosure_marshal_VOID__BOOLEAN,
                               G_TYPE_NONE,
                               1,
                               G_TYPE_BOOLEAN);
@@ -417,7 +417,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_STRUCT_OFFSET (PanelWidgetClass, end_move),
                               NULL,
                               NULL,
-                              panel_marshal_VOID__VOID,
+                              g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
 
@@ -524,12 +524,16 @@ panel_widget_reset_focus (GtkContainer *container,
 	PanelWidget *panel = PANEL_WIDGET (container);
 
 	if (container->focus_child == widget) {
-		GList *l;
+		GList *children;
+
+		children = gtk_container_get_children (container);
+
+		/* More than one element on the list */
+		if (children && children->next) {
+			GList *l;
 
-		l = gtk_container_get_children (container);
-		if (l && l->next) { /* More than one element on the list */
 			/* There are still object on the panel */
-			for (; l; l = l->next) {
+			for (l = children; l; l = l->next) {
 				GtkWidget *child_widget;
 
 				child_widget = l->data;
@@ -550,7 +554,7 @@ panel_widget_reset_focus (GtkContainer *container,
 		} else
 			panel_widget_focus (panel);
 
-		g_list_free (l);
+		g_list_free (children);
 	}
 }
 
